<?php

namespace Odhen\API\Service;

use \SimpleXMLElement;

class WiBX {

    protected $entityManager;
    protected $instanceManager;
    protected $sessionHandler;

    public function __construct(\Doctrine\ORM\EntityManager $entityManager, \Zeedhi\Framework\DependencyInjection\InstanceManager $instanceManager, \Helpers\Environment $sessionHandler){
        $this->entityManager = $entityManager;
        $this->instanceManager = $instanceManager;
        $this->sessionHandler = $sessionHandler;
    }

    const CALL_HEALTHCHECK = 'healthcheck';
    const CALL_QUOTATIONS = 'quotations';
    const CALL_PAYMENTS = 'payments';
    const CALL_REPORTS = 'reports';

    public function healthcheck(){
        $this->callWiBX(self::CALL_HEALTHCHECK, json_encode(array()), 'GET');
    }

    public function quotations($value, $saleCode){
        $params = array(
            'monetaryAmount' => $value,
            'referenceIdentifier' => $saleCode
        );

        return $this->callWiBX(self::CALL_QUOTATIONS, json_encode($params), 'POST');
    }

    public function payments($value, $saleCode, $type, $CDFILIAL, $CDCAIXA){
        if ($type == 'POST'){

            $params = array(
                'CDFILIAL' => $CDFILIAL,
                'CDCAIXA' => $CDCAIXA
            );
            $machineIdentifier = $this->entityManager->getConnection()->fetchAssoc("GET_WIBX_IDENTIFIER", $params);
            if ($machineIdentifier['NMCAIXAWIBX'] == null){
                throw new \Exception("ERRO:<br><br>O identificador WiBX não foi configurado para este caixa. Favor realizar a parametrização antes de efetuar pagamentos utilizando o WiBX.");
            }

            $params = array(
                'referenceIdentifier' => $saleCode,
                'wibxAmount' => $value,
                'machineIdentifier' => $machineIdentifier['NMCAIXAWIBX']
            );
            $params = json_encode($params);
        }
        else {
            $params = $saleCode;
        }

        return $this->callWiBX(self::CALL_PAYMENTS, $params, $type);
    }

    private function callWiBX($method, $params, $type){
        $URL = 'https://api.wiboo.io/qa-epp/v1/' . $method;

        $curl = curl_init();

        $apiKey = $this->sessionHandler->getUserInfo()['IDWIBXSELLER'];
        $wibxToken = $this->sessionHandler->getUserInfo()['IDWIBXTOKEN'];
        $enterpriseId = $this->sessionHandler->getUserInfo()['IDWIBXENTERP'];
        if (empty($apiKey) || empty($wibxToken) || empty($enterpriseId)){
            throw new \Exception("ERRO:<br><br>Uma ou mais das chaves necessárias para a utilização do WiBX não foram configuradas. Favor realizar a parametrização.");
        }

        $headers = array(
            'Content-Type: application/json',
            'x-api-key : ' . $apiKey
        );
        if ($method == self::CALL_QUOTATIONS || $method == self::CALL_PAYMENTS){
            array_push($headers, 'x-wibx-token : ' . $wibxToken);
            array_push($headers, 'x-enterprise-id : ' . $enterpriseId);
            if ($type !== 'POST'){
                $URL .= '/' . $params;
                if ($type == 'DELETE'){
                    curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'DELETE');
                    curl_setopt($curl, CURLOPT_POSTFIELDS, json_encode(array('description' => "Manual cancellation.")));
                }
            }
            else curl_setopt($curl, CURLOPT_POSTFIELDS, $params);
        }

        curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);

        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_TIMEOUT, 20);
        curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 10);

        curl_setopt($curl, CURLOPT_URL, $URL);

        if (!empty($this->instanceManager->getParameter('PROXY_IP'))){
            curl_setopt($curl, CURLOPT_PROXY, $this->instanceManager->getParameter('PROXY_IP').":".$this->instanceManager->getParameter('PROXY_PORT'));
            if (!empty($this->instanceManager->getParameter('PROXY_USER'))){
                curl_setopt($curl, CURLOPT_PROXYUSERPWD, $this->instanceManager->getParameter('PROXY_USER').":".$this->instanceManager->getParameter('PROXY_PASSWORD'));
            }
        }

        $curl_response = curl_exec($curl);
        $err    = curl_errno($curl);
        $errmsg = curl_error($curl);
        curl_close($curl);

        if ($err){
            if ($err === 28){
                throw new \Exception("<br>N&atilde;o foi poss&iacute;vel estabelecer uma conex&atilde;o com o servidor da WiBX.");
            }
            else {
                throw new \Exception("<br>Ocorreu um erro na chamada WiBX:<br><br>" . $errmsg);
            }
        }

        if (empty($curl_response)){
            throw new \Exception("<br>N&atilde;o houve retorno do servidor WiBX.");
        }
        else {
            $result = json_decode($curl_response, true);

            if (empty($result)){
                throw new \Exception($curl_response);
            }
            else if (array_key_exists('eventError', $result)){
                if (array_key_exists('message', $result)){
                    if (strpos($result['message'], 'The reference identifier already exists.')){
                        throw new \Exception("ERRO:<br><br>Uma transação WiBX com o número de sessão desta venda já foi requisitada.<br><br>Favor sair da tela de recebimento e realizar uma nova transação WiBX.");
                    }
                    if (strpos($result['message'], 'There is no machine with this name!')){
                        $params = json_decode($params, true);
                        throw new \Exception("ERRO:<br><br>O identificador WiBX registrado para este caixa (" . $params['machineIdentifier'] .") não foi reconhecido pela WiBX.<br><br>Favor verificar se o identificador está correto, ou entrar em contato com a WiBX para verificar a situação.");
                    }
                }
                throw new \Exception("<br>Erro na requisição WiBX: <br><br>" . $result['stacktrace']);
            }
            else if (array_key_exists('message', $result)){
                throw new \Exception("<br>Erro na requisição WiBX: <br><br>" . $result['message']);
            }

            return $result;
        }
    }

}